#!/usr/bin/env python
#Author: Mark Feineigle
#Create Date: 2018/04/27
#Last Modify: 2018/04/27
# deeper learning insights machine learning page 475

# KNN
from sklearn.neighbors import KNeighborsClassifier as knn
from sklearn import datasets
import numpy as np
import matplotlib.pyplot as plt
from matplotlib.colors import ListedColormap
def knnDemo(X,y, n):
  #creates the classifier and fits it to the data
  res = 0.05
  k1 = knn(n_neighbors=n, p=2, metric='minkowski')
  k1.fit(X, y)
  #sets up the grid
  x1_min, x1_max = X[:, 0].min() - 1, X[:, 0].max() + 1
  x2_min, x2_max = X[:, 1].min() - 1, X[:, 1].max() + 1
  xx1, xx2 = np.meshgrid(np.arange(x1_min, x1_max, res),
                         np.arange(x2_min, x2_max, res))
  #makes the prediction
  Z = k1.predict(np.array([xx1.ravel(), xx2.ravel()]).T)
  Z = Z.reshape(xx1.shape)
  #creates the color map
  cmap_light = ListedColormap(['#FFAAAA', '#AAFFAA', '#AAAAFF'])
  cmap_bold = ListedColormap(['#FF0000', '#00FF00', '#0000FF'])
  #Plots the decision surface
  plt.contourf(xx1, xx2, Z, alpha=0.4, cmap=cmap_light)
  plt.xlim(xx1.min(), xx1.max())
  plt.ylim(xx2.min(), xx2.max())
  #plots the samples
  for idx, cl in enumerate(np.unique(y)):
    plt.scatter(X[:, 0], X[:, 1], 
                c=y,
                cmap=cmap_bold)
  plt.show()

iris = datasets.load_iris()
X1 = iris.data[:,0:3:2]
X2 = iris.data[:,0:2]
X3 = iris.data[:,1:3]
y = iris.target
knnDemo(X2,y,15)


# RIDGE
from sklearn.linear_model import Ridge
import numpy as np
def ridgeReg(alpha):
  n_samples, n_features = 10, 5
  y = np.random.randn(n_samples)
  X = np.random.randn(n_samples, n_features)
  clf = Ridge(.001)
  res = clf.fit(X, y)
  return(res)
res = ridgeReg(0.001)
print(res.coef_)
print(res.intercept_)


# KERNEL PCA
import numpy as np
import matplotlib.pyplot as plt
from sklearn.decomposition import KernelPCA #principle component analysis
from sklearn.datasets import make_circles
np.random.seed(0)
X, y = make_circles(n_samples=400, factor=.3, noise=.05)
kpca = KernelPCA(kernel='rbf', gamma=10) #radial basis function
X_kpca = kpca.fit_transform(X)
plt.figure()
plt.subplot(2, 1, 1, aspect='equal')
plt.title("Original space")
reds = y == 0
blues = y == 1
plt.plot(X[reds, 0], X[reds, 1], "ro")
plt.plot(X[blues, 0], X[blues, 1], "bo")
plt.xlabel("$x_1$")
plt.ylabel("$x_2$")
plt.subplot(2, 1, 2, aspect='equal')
plt.plot(X_kpca[reds, 0], X_kpca[reds, 1], "ro")
plt.plot(X_kpca[blues, 0], X_kpca[blues, 1], "bo")
plt.title("Projection by KPCA")
plt.xlabel("1st principal component in space induced by $\phi$")
plt.ylabel("2nd component")
plt.subplots_adjust(0.02, 0.10, 0.98, 0.94, 0.04, 0.35)
plt.show()
#print('gamma= %0.2f' % gamma)


# CROSS VALIDATION
from sklearn.cross_validation import train_test_split
from sklearn import datasets
from sklearn import svm
from sklearn import cross_validation
iris = datasets.load_iris()
X_train, X_test, y_train, y_test = train_test_split (iris.data,
                                                     iris.target,
                                                     test_size=0.4,
                                                     random_state=0)
clf = svm.SVC(kernel='linear', C=1)
clf.fit(X_train, y_train)
scores = cross_validation.cross_val_score(clf, X_train, y_train, cv=5)
print("Accuracy: %0.2f (+/- %0.2f)" % (scores.mean(), scores.std() * 2))


# MEAN, GAUSSIAN
import numpy as np
import matplotlib.pyplot as plt
import matplotlib.mlab as mlab
def normal(mean = 0, var = 1): #var is std**2
  sigma = np.sqrt(var)
  x = np.linspace(-3, 3, 100)
  plt.plot(x, mlab.normpdf(x, mean, sigma))
  plt.show()
normal(1, 0.5)


# BINOMIAL DISTRIBUTION
from scipy.stats import binom
def binomial(x=10, n=10, p=0.5):
  fig, ax = plt.subplots(1, 1)
  x = range(x)
  rv = binom(n, p)
  plt.vlines(x, 0, (rv.pmf(x)), colors='k', linestyles='-')
  plt.show()
binomial()


# POISSON
from scipy.stats import poisson
def pois(x=1000):
  xr = range(x)
  ps = poisson(xr)
  plt.plot(ps.pmf(x/2))
  plt.show()
pois()


# CUMULATIVE DENSITY FUNCTION
import scipy.stats as stats
def cdf(s1=50,s2=0.2):
  x = np.linspace(0,s2 * 100,s1 *2)
  cd = stats.binom.cdf
  plt.plot(x,cd(x, s1, s2))
  plt.show()
cdf()

